/****************************************************************************
                          tool_timetools.h
						  -------------------
    begin                : Fri Dec 08 2006
    authors              : Xiang Li
	email                : xiangli@ieee.org
****************************************************************************/

#ifndef __TOOL_TIMETOOLS_H__
#define __TOOL_TIMETOOLS_H__

/****************************************************************************
    Implementation of tools for time information.
****************************************************************************/

/** standard headers *******************************************************/
#include <time.h>
#ifdef WIN32
	#include "Windows.h"
#else
	#include <sys/time.h> 
#endif
/***************************************************************************/

/***************************************************************************/
namespace Utilities
{

//! structure for timing 
typedef struct _STimeRecord
{
	//! elapsed time in millisecond
	double	dElapsedTime;

	//! kernel time in millisecond
	double	dKernelTime;

	//! user time in millisecond
	double	dUserTime;

	//! number of being called
	unsigned int uNumCalled;

	//! number of actual timing
	unsigned int uNumTiming;

	//! accumulated elapsed time in millisecond
	double	dAccuElapsedTime;

	//! accumulated kernel time in millisecond
	double	dAccuKernelTime;

	//! accumulated user time in millisecond
	double	dAccuUserTime;

}STimeRecord;

//! class for time tools
class CTimeTools
{
public:

/** function members *******************************************************/

	//! constructor
	CTimeTools();

	//! destructor
	~CTimeTools();

	/***********************************************************************/
	  /*!
	   * \short Reset the timer
	   */
	/***********************************************************************/
	void Reset();

	/***********************************************************************/
	  /*!
	   * \short Get the current time (and date) information
	   *
	   * \return a string contains the time information
	   */
	/***********************************************************************/
	const char * GetTimeString() const;

	/***********************************************************************/
	  /*!
	   * \short Start timing for a specified process
	   *
	   * \param bNoTiming, whether do the real timing (only count calling times when true)
	   */
	/***********************************************************************/
	void StartTiming( bool bNoTiming = false );

	/***********************************************************************/
	  /*!
	   * \short Stop timing and get the duration in millisecond
	   *
	   * \param bNoTiming, whether do the real timing (only count calling times when true)
	   *
	   * \return a constant reference to the time information
	   */
	/***********************************************************************/
	const STimeRecord & StopTiming( bool bNoTiming = false );

	/***********************************************************************/
	  /*!
	   * \short Get the current record
	   *
	   * \return a constant reference to the time information
	   */
	/***********************************************************************/
	const STimeRecord & GetRecord() const
	{
		return( m_TimingRecord );
	}

/***************************************************************************/

private:

/** variable members *******************************************************/

	//! internal timing structures
#ifdef WIN32
	typedef struct _SProcessTiming
	{
		FILETIME tElapsed;

		FILETIME tKernel;

		FILETIME tUser;

	}SProcessTiming;	

	LARGE_INTEGER m_nFrequency;

#else
	typedef struct _SProcessTiming
	{
		timeval	tElapsed;

		timeval tSystem;

		timeval tUser;

	}SProcessTiming;	
#endif

	//! timing records
	STimeRecord m_TimingRecord;

	//! start record
	SProcessTiming m_TimingStart;

	//! end record
	SProcessTiming m_TimingEnd;

#if _DEBUG

	//! flag to detect whether there is an embedded timing
	bool m_bStart;

#endif

/***************************************************************************/

/** function members *******************************************************/

	/***********************************************************************/
	  /*!
	   * \short Time duration between the two time point
	   *
	   * \param pStart, the start time point
	   * \param pEnd, the end time point
	   *
	   * \return the time between the start point and end point, in millisecond
	   */
	/***********************************************************************/
#ifdef WIN32
	inline double xxTimeDuration( ULARGE_INTEGER * pStart , ULARGE_INTEGER * pEnd ) const
	{
		double dDuration = ( double )( pEnd->QuadPart - pStart->QuadPart ); // in the unit of 100-nanosecond 
		return( dDuration / 10000.0 ); 
	}

	inline double xxTimeDuration( LARGE_INTEGER * pStart , LARGE_INTEGER * pEnd ) const
	{
		double dDuration = ( double )( pEnd->QuadPart - pStart->QuadPart ); 
		return( 1000 * dDuration / m_nFrequency.QuadPart ); 
	}
#else

#endif	
/***************************************************************************/
};

} // namespace Utilities
/***************************************************************************/

#endif //__TOOL_TIMETOOLS_H__

/****************************************************************************
    end-of-file
****************************************************************************/
